/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.                                    
 *                                                                 
 * The Initial Developer of the Original Code is Karsten Duesterloh.
 *                                                                 
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.                     
 *                                                                 
 * Contributor(s):                                                 
 *  Karsten Duesterloh <mnenhy@tprac.de>                            
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
// +-
// |    SourceRDF.js
// |
// |    Written, forged, copied and conceived by Karsten Duesterloh <mnenhy@tprac.de>
// |
// |    This file is inspired by 
// |      MozTweakRDF.js by Marek Wawoczny <Marek.Wawoczny@GmbH.rubikon.pl>, and
// |      pRDFData.js by SHIMODA Hiroshi <piro@p.club.ne.jp>
// |    But there isn't too much code left anyway that has survived my editing... ;-)
// +-    

var SOURCERDF_MNENHY_LOAD = true;  // for include

// All container dependend operations will be relative to $RDFBaseURI:$BaseContainer;
// $RDFBaseURI:$BaseContainer will be created upon instantiiation if necessary
// Use SetBaseContainer to change it afterwards.
// The rdf about attributes will have the form
//    $RDFBaseURI:$BaseContainer[:$SubContainer]*[:$Item]
// where $BaseContainer = $Container [:$Container]*
// and   $SubContainer  = $Container [:$Container]* 
// Therefore a $BaseContainer "A" and a $SubContainer "B:C" are equal to a
// $BaseContainer "A:B" and a $SubContainer "C"!
// 


// pseudo constants
cBitSourceRDF_Clear = 0x00;
cBitSourceRDF_Load  = 0x01;
cBitSourceRDF_Halt  = 0x02;
cBitSourceRDF_Ready = 0x04;
cBitSourceRDF_Error = 0x08;


// Constructor
function SourceRDF(
    /* nsIFile  */ oDataSourceFile,                   // rdf-file [URI is created from this]
    /* string   */ sDataNameSpace     /* = ""     */, // namespace for elements inside of rdf:description
    /* string   */ sRDFBaseURI        /* = "urn"  */, // base URI for about-attributes
    /* string   */ sBaseContainer     /* = "data" */,
    /* string   */ sBaseContainerType /* = "Seq"  */,
    /* function */ fNotifier          /* = DefaultNotifier(oSourceRDF) */,
                                                      // fNotifier(this) if datasource is ready;
                                                      // fNotifier(null) in case of error
    /* nsIFile  */ oDefaultSourceFile /* = null   */, // default rdf file
    /* boolean  */ bBlocking          /* = false  */  // if true, rdf is opened synchronously
)
{
  this.nsIRDFService           = Components.classes["@mozilla.org/rdf/rdf-service;1"]    .getService().QueryInterface(Components.interfaces.nsIRDFService);
  this.nsIRDFContainer         = Components.classes["@mozilla.org/rdf/container;1"]      .getService().QueryInterface(Components.interfaces.nsIRDFContainer);
  this.nsIRDFContainerUtils    = Components.classes["@mozilla.org/rdf/container-utils;1"].getService().QueryInterface(Components.interfaces.nsIRDFContainerUtils);

  this.mBlocking               = bBlocking || false;
  this.mDataSource             = null;
  this.mDataSourceLoaded       = false;
  this.mDataSourceCreated      = false;   // only true, if new rdf file has been created
  this.mDataSourceState        = cBitSourceRDF_Clear;
  this.mDataSourceURI          = this.getDataSourceURI(oDataSourceFile, oDefaultSourceFile);
  this.mSinkObserverRegistered = false;
  this.mNotifier               = fNotifier      || DefaultNotifier;

  this.mDataNameSpace          = sDataNameSpace || "";
  this.mRDFBaseURI             = sRDFBaseURI    || "urn";
  this.SetBaseContainer(sBaseContainer, sBaseContainerType);

  this.LoadDataSource();
}

  
// the prototype for the RDF object
SourceRDF.prototype = 
{ //
  // datasource functions
  //
  LoadDataSource: function() 
  { // we only need to load the datasource if it hasn't been done already
    if(!this.mDataSource || !this.mDataSourceLoaded || !this.mSinkObserverRegistered)
    {
      // synchronous or asynchronous?
      if (this.mBlocking) 
      {
        this.mDataSource = this.nsIRDFService.GetDataSourceBlocking(this.mDataSourceURI);
      }
      else 
      {
        this.mDataSource = this.nsIRDFService.GetDataSource(this.mDataSourceURI);
      }

      var sinkObserver = 
      {
        RDF:          this,
        onBeginLoad:  function(aSink) 
                      { 
                        this.RDF.mDataSourceState = (this.RDF.mDataSourceState | cBitSourceRDF_Load); 
                      },
        onInterrupt:  function(aSink) 
                      { 
                        this.RDF.mDataSourceState = (this.RDF.mDataSourceState | cBitSourceRDF_Halt); 
                      },
        onResume:     function(aSink) 
                      { 
                        this.RDF.mDataSourceState = (this.RDF.mDataSourceState & ~cBitSourceRDF_Halt);
                      },
        onError:      function(aSink, aStatus, aErrorMsg) 
                      { 
                        this.RDF.mDataSourceState = (this.RDF.mDataSourceState | cBitSourceRDF_Error);
                        //alert(aStatus + ": " + aErrorMsg);
                        this.RDF.mNotifier(null);
                      },
        onEndLoad:    function(aSink) 
                      { 
                        this.RDF.mDataSourceState = (this.RDF.mDataSourceState | cBitSourceRDF_Ready);
                        this.RDF.mDataSourceLoaded = (this.RDF.mDataSourceState == (cBitSourceRDF_Load+cBitSourceRDF_Ready));
                        this.RDF.mNotifier(this.RDF);
                      }
      }

      if (this.mDataSource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).loaded) 
      {
        this.mDataSourceLoaded = (this.mDataSource != null);
        this.mNotifier(this);
      }
      else 
      { // register sink observer
        this.mDataSourceLoaded       = false;
        this.mDataSourceState        = 0x0;
        this.mSinkObserverRegistered = true;
        this.mDataSource.QueryInterface(Components.interfaces.nsIRDFXMLSink).addXMLSinkObserver(sinkObserver);
      }
    }
  },

  IsLoading: function()
  { // no error, no ready, but loading
    return ( (this.mDataSourceState & ~cBitSourceRDF_Halt) == cBitSourceRDF_Load);
  },
  IsGood: function()
  { // data source loaded successfully
    return this.mDataSourceLoaded;
  },
  IsBad: function()
  { // error
    return ( (this.mDataSourceState & cBitSourceRDF_Error) != cBitSourceRDF_Clear);
  },

  RefreshDataSource: function() 
  {
    this.mDataSource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Refresh(false);
  },


  FlushDataSource: function() 
  {
    this.mDataSource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
  },

  
  //
  // container functions
  //
  
  // set the default container
  SetBaseContainer: function(sBaseContainer, sBaseContainerType)
  {
    this.mBaseContainer     = sBaseContainer || "data";
    this.mBaseContainerType = this.GetContType(sBaseContainerType);
  },
  

  // get the container element count
  ContainerSize: function(sContainer)
  {
    var iSize = 0;
    if (!sContainer)
      sContainer = "";
    if (sContainer.length && this.InitContainer(sContainer))
      iSize = this.nsIRDFContainer.GetCount();
    return iSize;
  },


  // create a new container
  // no sContainer       -> create mBaseContainer, ignore sParentContainer
  // no sContainerType   -> set to sBaseContainerType
  // no sParentContainer -> set as child of BaseContainer
  AddContainer: function(sContainer, sContainerType, sParentContainer) 
  { 
    if (!sContainer)
      sContainer = "";
  
    // check parent
    var pares = null;
    if (sContainer.length)
    { 
      if (this.InitContainer(sParentContainer))
      { // parent exists, so prepend it to new container name if not BaseContainer
        if (sParentContainer && sParentContainer.length)
          sContainer = sParentContainer + ":" + sContainer;
      }
      else
      { // invalid parent -> no child
        return;
      }
    }    

    var contres = this.GetContRes(sContainer);
    switch (this.GetContType(sContainerType))
    {
      case "Alt":
				this.nsIRDFContainerUtils.MakeAlt(this.mDataSource, contres);
				break;
			case "Bag":
				this.nsIRDFContainerUtils.MakeBag(this.mDataSource, contres);
				break;
			case "Seq":
			default:
				this.nsIRDFContainerUtils.MakeSeq(this.mDataSource, contres);
				break;
		}

    // add new container to parent if necessary
    if ( (sContainer.length) && (this.nsIRDFContainer.IndexOf(contres)==-1) )
    {
      this.nsIRDFContainer.AppendElement(contres);
    }
	},


  // remove all outgoing arcs from oElement
  // oElement must implement nsIRDFResource
  RemoveArcsOut: function(oElement)
  {
    oElement = oElement.QueryInterface(Components.interfaces.nsIRDFResource);

    var ArcLabelsOut = this.mDataSource.ArcLabelsOut(oElement);
    while(ArcLabelsOut.hasMoreElements()) 
    {
      var Arc = ArcLabelsOut.getNext();
      var Targets = this.mDataSource.GetTargets(oElement, Arc, true);
      while (Targets.hasMoreElements()) 
      {
        var Target = Targets.getNext();

        // debug output!
        /*
        var et = oElement;try{et = et.QueryInterface(Components.interfaces.nsIRDFResource).Value}catch(i){};
        var at = Arc;     try{at = at.QueryInterface(Components.interfaces.nsIRDFResource).Value}catch(i){};
        var tt = Target;  try{tt = tt.QueryInterface(Components.interfaces.nsIRDFResource).Value}catch(i){};
        alert("RemoveArcsOut("+oElement.value+")\n E="+et+"\n A="+at+"\n T="+tt);
        */

        this.mDataSource.Unassert(oElement, Arc, Target, true);
      }
    }
  }, 
  
  
  // remove container with all its elements
  // the Container param can be either the name string
  // or any object that can be casted to nsIRDFResource
  RemoveContainer: function(Container) 
  { // check existence
    var contres = this.GetContRes(Container);
    if (!this.InitContainer(contres))
      return; // done!

    // loop over all the container's elements
    var Elements = this.nsIRDFContainer.GetElements();
    while(Elements.hasMoreElements()) 
    {
      var Element = Elements.getNext();
      if(this.nsIRDFContainerUtils.IsContainer(this.mDataSource, Element)) 
      { // recur subcontainers
        this.RemoveContainer(Element);
        this.nsIRDFContainer.Init(this.mDataSource, contres); // reinit
      }

      // remove all outgoing arcs from Element
      this.RemoveArcsOut(Element);
      this.nsIRDFContainer.RemoveElement(Element, false);
    }

    // remove all arcs from this container
    // (this is a workaround to get rid of main level containers)
    this.RemoveArcsOut(contres);
  },
  

  //
  // node functions
  //  

  // check if Node exists
  ExistNode: function(sNode, sContainer)
  {
    var bOK = this.InitContainer(sContainer);
    if (bOK)
    { 
      var item = this.GetDescRes(sNode, sContainer);
      bOK = (this.nsIRDFContainer.IndexOf(item) != -1);
    }
    return bOK;
  },  
  
  
  // add node sNode to sContainer and fill with all elements of the *associative* Array aElementArray
  // sContainer defaults to mBaseContainer
  AddNode: function(sNode, sContainer, aElementArray) 
  {
    if (this.InitContainer(sContainer))
    { 
      var item = this.GetDescRes(sNode, sContainer);
      if (this.nsIRDFContainer.IndexOf(item)==-1)
        this.nsIRDFContainer.AppendElement(item);
      for (var elem in aElementArray) 
      {
        this.ChangeItemValue(aElementArray[elem],
                             elem,
                             sNode,
                             sContainer);
      }

    }
  },


  // remove the node sNode of sContainer
  RemoveNode: function(sNode, sContainer, bSkipInitContainer)
  {
    if (IsTrue(bSkipInitContainer) || this.InitContainer(sContainer))
    {
      var item = this.GetDescRes(sNode, sContainer);
      this.RemoveArcsOut(item);
      this.nsIRDFContainer.RemoveElement(item, false);
    }
  },


  // get a list with all the container's node names
  // if given, the list is restricted to nodes with sItem set to sValue
  // returns an array of strings (associative, if bAssocArray = true)
  GetNodeList: function(sContainer, bAssocArray, sItem, sValue)
  {
    var aNodeList = new Array();
    var sValue = String(sValue);

    if (this.InitContainer(sContainer))
    {
      var nsISimpleEnumerator = this.nsIRDFContainer.GetElements();
      while (nsISimpleEnumerator.hasMoreElements())
      {
        var oNode    = nsISimpleEnumerator.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
        var bGetNode = !(sItem && sValue && sItem.length && sValue.length);

        if (!bGetNode)
        { // check the item value
          try
          {
            var target = this.mDataSource.GetTarget(oNode, 
                                                    this.GetItemRes(sItem),
                                                    true);
            if (target)
            {
              var sRead = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
              bGetNode = (String(sRead) == String(sValue));
            }
          }
          catch(ignore){}
        }

        if (bGetNode)
        {
          var sNodeName = oNode.Value;
          sNodeName = sNodeName.slice(sNodeName.lastIndexOf(":") + 1);
          var sElem = bAssocArray ? sNodeName : aNodeList.length;
          aNodeList[sElem] = sNodeName;
        }
      } // while
    } // if container init ok
    return aNodeList;
  },


  //
  // item functions
  //

  // if not given, sContainer defaults to mBaseContainer
  GetItemValue: function(sItem, sNode, sContainer) 
  {
    var value = null;
    if (this.InitContainer(sContainer))
    { 
      try
      {
        var target = this.mDataSource.GetTarget(this.GetDescRes(sNode, sContainer), 
                                                this.GetItemRes(sItem),
                                                true);
        if (target)
          value = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
      }
      catch(ignore)
      { /* return only default null value */ }
    }
    return value;
  },


  // if not given, sContainer defaults to mBaseContainer
  ChangeItemValue: function(newValue, sItem, sNode, sContainer) 
  {
    if (this.InitContainer(sContainer))
    { 
      var oldValue = this.GetItemValue(sItem, sNode, sContainer);
      if (oldValue) 
        oldValue = this.GetLit(oldValue);
      if (newValue) 
        newValue = this.GetLit(newValue);

      if (oldValue || newValue)
      {
        if (oldValue && !newValue) 
        {
          this.mDataSource.Unassert(this.GetDescRes(sNode, sContainer),
                                    this.GetItemRes(sItem),
                                    oldValue);
        }
        else if (!oldValue && newValue) 
        {
          this.mDataSource.Assert  (this.GetDescRes(sNode, sContainer),
                                    this.GetItemRes(sItem),
                                    newValue, 
                                    true);
        }
        else 
        {
          this.mDataSource.Change  (this.GetDescRes(sNode, sContainer),
                                    this.GetItemRes(sItem),
                                    oldValue, 
                                    newValue);
        }
      }
    }
  },

 
  //
  // some convenience functions
  //

  GetRes: function(sResName)
  {
    return this.nsIRDFService.GetResource(sResName);
  },
  GetLit: function(sLiteral)
  {
    return this.nsIRDFService.GetLiteral(sLiteral);
  },
  
  // convert to container resource
  // - string params need BaseURI and BaseContainer
  // - object params are queried for their nsIRDFResource.Value
  GetContRes: function(contres)
  {
    if (!contres)
      contres = "";

    if (typeof(contres)=="string")
    {
      var contname = this.mRDFBaseURI + ":" + this.mBaseContainer;
      if (contres.length)
        contname += ":" + contres;
      contres = contname;
    }
    else
    {
      contres = contres.QueryInterface(Components.interfaces.nsIRDFResource).Value;
    }
    return this.GetRes(contres);
  },

  // convert to description (node) resource
  GetDescRes: function(sNode, sContainer)
  {
    return this.GetRes(       this.mRDFBaseURI 
                        + ":" + this.mBaseContainer 
                        + ( (sContainer && sContainer.length) ? (":" + sContainer) : ("") )
                        + ( (sNode      && sNode.length     ) ? (":" + sNode     ) : ("") )
                      );
  },

  // convert to item resource
  GetItemRes: function(sItem)
  {
    return this.GetRes(this.mDataNameSpace + sItem);
  },

  // try to guess the container type from string
  GetContType: function(sType)
  {
    if (sType)
      sType = sType.toLowerCase();
    switch (sType)
    {
      case "alt":
        return "Alt";
      case "bag":    
        return "Bag";
      case "seq":
      default:
        return "Seq";
    }
  },  

  // check existence of container via nsIRDFContainer.Init
  InitContainer: function(sContainer, bQuiet) 
  {
    try
    {
      this.nsIRDFContainer.Init(this.mDataSource, this.GetContRes(sContainer));
    }
    catch(error)
    { // we usually get here if sContainer does not exist
      if (!IsTrue(bQuiet))
        alert("Fatal Error in SourceRDF::InitContainer(["+this.GetContRes(sContainer).Value+"])\n"+error);
      return false;
    }
    return true;
  },
    
  
  // create URI of rdfFile
  // if rdfFile does not exist, try copying default
  // if no default is given, try creating new oDataSourceFile
  getDataSourceURI: function(oSource, oDefault)
  {
    if (!oSource)
      return null;
    
    try
    {
      if (!oSource.exists())  // rdf not found
      { // maybe we can copy it from somewhere?
        var bCreate = !oDefault || !oDefault.exists();
        if (!bCreate)
        { // copy it
          oDefault.copyTo(oSource.parent, oSource.leafName);
          bCreate = !oSource.exists();
        }

        if (bCreate)
        { // create new oSource
          var clNORMAL_FILE_TYPE = 0;     // file
          var clPermissions      = 0644;  // only owner may change, others only read
          oSource.create(clNORMAL_FILE_TYPE, clPermissions)
        }
        this.mDataSourceCreated = true;
      } 
      var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
      return ioService.newFileURI(oSource).spec;
    }
    catch (err)
    {
      return null;
    }
  }
};

// default notifier function
// gets called if loading datasource is finished
function DefaultNotifier(oSourceRDF)
{
  // do nothing
}

