/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Duesterloh.
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Duesterloh <mnenhy@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// +-
// |    mnenhy-junk-stats.js
// |
// |    Written, forged, copied and conceived by Karsten Duesterloh <mnenhy@tprac.de>
// |    Inspired by Moztweak-ROT13 by Marek Wawoczny <Marek.Wawoczny@GmbH.rubikon.pl>
// |         and by Kenny-Speak    by Kohan Ikin
// +-

//
// global arrays for easy encoding
//

const gasROT13 = {a: "n",  b: "o",  c: "p",  d: "q",  e: "r",  f: "s",  g: "t",  h: "u",  i: "v",
                  j: "w",  k: "x",  l: "y",  m: "z",  n: "a",  o: "b",  p: "c",  q: "d",  r: "e",
                  s: "f",  t: "g",  u: "h",  v: "i",  w: "j",  x: "k",  y: "l",  z: "m",
                  A: "N",  B: "O",  C: "P",  D: "Q",  E: "R",  F: "S",  G: "T",  H: "U",  I: "V",
                  J: "W",  K: "X",  L: "Y",  M: "Z",  N: "A",  O: "B",  P: "C",  Q: "D",  R: "E",
                  S: "F",  T: "G",  U: "H",  V: "I",  W: "J",  X: "K",  Y: "L",  Z: "M"          };
const gasKenny = {  // normal to Kenny
                  a:"mmm", b:"mmp", c:"mmf", d:"mpm", e:"mpp", f:"mpf", g:"mfm", h:"mfp", i:"mff",
                  j:"pmm", k:"pmp", l:"pmf", m:"ppm", n:"ppp", o:"ppf", p:"pfm", q:"pfp", r:"pff",
                  s:"fmm", t:"fmp", u:"fmf", v:"fpm", w:"fpp", x:"fpf", y:"ffm", z:"ffp",
                  "":"mmmmpp",     "":"ppfmpp",     "":"fmfmpp",     "":"fmmffp",
                  A:"Mmm", B:"Mmp", C:"Mmf", D:"Mpm", E:"Mpp", F:"Mpf", G:"Mfm", H:"Mfp", I:"Mff",
                  J:"Pmm", K:"Pmp", L:"Pmf", M:"Ppm", N:"Ppp", O:"Ppf", P:"Pfm", Q:"Pfp", R:"Pff",
                  S:"Fmm", T:"Fmp", U:"Fmf", V:"Fpm", W:"Fpp", X:"Fpf", Y:"Ffm", Z:"Ffp",
                  "":"Mmmmpp",     "":"Ppfmpp",     "":"Fmfmpp",
                    // Kenny to normal
                  mmm:"a", mmp:"b", mmf:"c", mpm:"d", mpp:"e", mpf:"f", mfm:"g", mfp:"h", mff:"i",
                  pmm:"j", pmp:"k", pmf:"l", ppm:"m", ppp:"n", ppf:"o", pfm:"p", pfp:"q", pff:"r",
                  fmm:"s", fmp:"t", fmf:"u", fpm:"v", fpp:"w", fpf:"x", ffm:"y", ffp:"z",
                  Mmm:"A", Mmp:"B", Mmf:"C", Mpm:"D", Mpp:"E", Mpf:"F", Mfm:"G", Mfp:"H", Mff:"I",
                  Pmm:"J", Pmp:"K", Pmf:"L", Ppm:"M", Ppp:"N", Ppf:"O", Pfm:"P", Pfp:"Q", Pff:"R",
                  Fmm:"S", Fmp:"T", Fmf:"U", Fpm:"V", Fpp:"W", Fpf:"X", Ffm:"Y", Ffp:"Z"         };
const gasMorse = {  // normal to Morse
                  "a":".-",       "b":"-...",     "c":"-.-.",     "d":"-..",      "e":".",
                  "f":"..-.",     "g":"--.",      "h":"....",     "i":"..",       "j":".---",
                  "k":"-.-",      "l":".-..",     "m":"--",       "n":"-.",       "o":"---",
                  "p":".--.",     "q":"--.-",     "r":".-.",      "s":"...",      "t":"-",
                  "u":"..-",      "v":"...-",     "w":".--",      "x":"-..-",     "y":"-.--",
                  "z":"--..",
                  "1":".----",    "2":"..---",    "3":"...--",    "4":"....-",    "5":".....",
                  "6":"-....",    "7":"--...",    "8":"---..",    "9":"----.",    "0":"-----",
                  ",":"--..--",   ":":"---...",   "?":"..--..",   "'":".----.",   '"':".-..-.",
                  "/":"-..-.",    "+":".-.-.",    "=":"-...-",    "(":"-.--.-",
                    // Morse to normal
                  ".-"    :"a",   "-..."  :"b",   "-.-."  :"c",   "-.."   :"d",   "."     :"e",
                  "..-."  :"f",   "--."   :"g",   "...."  :"h",   ".."    :"i",   ".---"  :"j",
                  "-.-"   :"k",   ".-.."  :"l",   "--"    :"m",   "-."    :"n",   "---"   :"o",
                  ".--."  :"p",   "--.-"  :"q",   ".-."   :"r",   "..."   :"s",   "-"     :"t",
                  "..-"   :"u",   "...-"  :"v",   ".--"   :"w",   "-..-"  :"x",   "-.--"  :"y",
                  "--.."  :"z",
                  ".----" :"1",   "..---" :"2",   "...--" :"3",   "....-" :"4",   "....." :"5",
                  "-...." :"6",   "--..." :"7",   "---.." :"8",   "----." :"9",   "-----" :"0",
                  ".-.-.-":".",   "--..--":",",   "---...":":",   "..--..":"?",   ".----.":"'",
                  ".-..-.":'"',   "-....-":"-",   "-..-." :"/",   ".-.-." :"+",   "-...-" :"=",
                  "-.--.-":"(",
                    // special chars
                  "dot"  : ".-.-.-",
                  "minus": "-....-",
                  "\n"   : "\n",
                  "\r"   : "\r",
                  "\n\r" : "\n\r",
                  "\t"   : "\t",
                  "\v"   : "\v",
                  "\f"   : "\f",
                  " "    : "",
                    // unreversable characters
                  ")":"-.--.-",     // coded as "("
                  "":".- .",       // coded as "ae"
                  "":"--- .",      // coded as "oe"
                  "":"..- .",      // coded as "ue"
                  "":"... --.."    // coded as "sz"
                 };

//
// functions
//

function MnenhyCodec(oCaller, fCodec)
{ // differentiate between editor and messagepane
  switch (oCaller.parentNode.parentNode.parentNode.id)
  {
    case "msgComposeContext":       MnenhyCallFromEditor (fCodec, "content-frame"); break;
    case "messagePaneContext":      MnenhyCallFromBrowser(fCodec, "messagepane"  ); break;
    case "viewSourceContextMenu":   MnenhyCallFromBrowser(fCodec, "content"      ); break;
    case "contentAreaContextMenu":  MnenhyCallFromBrowser(fCodec, "content"      ); break;
  }
}


function MnenhyCallFromEditor(fCodec, sEditorID)
{
  var messageContentFrame  = document.getElementById(sEditorID); // <editor>
  var messageComposeEditor = messageContentFrame.getEditor(messageContentFrame.contentWindow);
  if (messageComposeEditor)
  {
    messageComposeEditor.beginTransaction();
    MnenhyProcessCodec(fCodec,
                       messageComposeEditor.rootElement,
                       messageComposeEditor.selection,
                       messageComposeEditor);
    messageComposeEditor.endTransaction();
  }
}


function MnenhyCallFromBrowser(fCodec, sBrowserID)
{
  var messageContentFrame = document.getElementById(sBrowserID); // <browser>
  if (messageContentFrame)
  {
    MnenhyProcessCodec(fCodec,
                       messageContentFrame.contentWindow.document.getElementsByTagName("body")[0],
                       messageContentFrame.contentWindow.getSelection());
  }
}


// apply codec to oTextNode
// If oEditor is specified, the oSelection is set to (oTextNode, iStartOffset, iEndOffset)
//   and changed by insertText to allow for undo. This will destroy the current selection!
// Otherwise, the node text is changed directly via replaceData.
// Returns new iEndOffset or -1 if unchanged.
function MnenhyApplyCodec(oTextNode,
                          fCodec,
                          oSelection,
                          iStartOffset /* = null */,
                          iEndOffset   /* = null */,
                          oEditor      /* = null */)
{
  if (iStartOffset == null)
    iStartOffset = 0;
  if (iEndOffset == null)
    iEndOffset = oTextNode.nodeValue.length;
  var iEndOffsetRev = iEndOffset;
  if (iEndOffset > iStartOffset)
  {
    var iOldLength = iEndOffset - iStartOffset;
    var sNewText = fCodec(oTextNode.nodeValue.substring(iStartOffset, iEndOffset));
    //dump("["+iStartOffset+","+iEndOffset+"]"+oTextNode.nodeValue+"\n==>"+sNewText+"\n");
    if (oEditor)
    { // set the selection to just the node
      oSelection.collapse(oTextNode, iStartOffset);
      oSelection.extend  (oTextNode, iEndOffset  );
      oEditor.insertText(sNewText);
    }
    else
    { // just change the text - no editor, no undo
      oTextNode.replaceData(iStartOffset, iEndOffset - iStartOffset, sNewText);
    }

    // set new iEndOffset
    if (sNewText.length != iOldLength)
      iEndOffsetRev += sNewText.length - iOldLength;
  }
  return iEndOffsetRev;
}


// return the index in the parent's childNodes
// (we expect its existence!)
function MnenhyFindIndexInParent(oNode)
{
  var oh = oNode.parentNode.childNodes;
  for (var ih = 0; ih < oh.length; ++ih)
  {
    if (oh[ih] == oNode)
      return ih;
  }
  return -1;  // error
}


function MnenhyProcessCodec(fCodec, oRootElement, oSelection, oEditor /* = null */)
{ // it isn't possible to have multiple ranges in MailNews
  var oTreeWalker = null;
  var oNextNode   = null;
  var oThisNode   = null;

  if (oSelection.rangeCount == 1 && !oSelection.isCollapsed)
  { // only touch the selected text
    var oRange = oSelection.getRangeAt(0);

    // remember the selection boundaries
    var oh, ih; // helper vars

    var oStartContainer = oRange.startContainer;
    var oStartParent    = oStartContainer.parentNode;
    var iStartOffset    = oRange.startOffset;
    var iStartIndex     = MnenhyFindIndexInParent(oStartContainer);

    var oEndContainer   = oRange.endContainer;
    var oEndParent      = oEndContainer.parentNode;
    var iEndOffset      = oRange.endOffset;
    var iEndOffsetRev   = iEndOffset;
    var iEndIndex       = MnenhyFindIndexInParent(oEndContainer);

    if ((oStartContainer == oEndContainer) && (oStartContainer.nodeType == Node.TEXT_NODE))
    { // just one text node
      iEndOffsetRev = MnenhyApplyCodec(oStartContainer, fCodec, oSelection, iStartOffset, iEndOffset, oEditor);
    }
    else
    { // different boundary containers or not text nodes

      // process the boundaries first, because the oTreeWalker may destroy them
      if ((oStartContainer.nodeType == Node.TEXT_NODE) && (iStartOffset > 0))
      { // start lies within TEXT_NODE
        MnenhyApplyCodec(oStartContainer, fCodec, oSelection, iStartOffset, null, oEditor);
      }
      if (oEndContainer.nodeType == Node.TEXT_NODE)
      { // set the iEndOffsetRev
        iEndOffsetRev = -1; // recalculate after oTreeWalker
        if (iEndOffset < oEndContainer.nodeValue.length)
        { // end lies within TEXT_NODE
          iEndOffsetRev = MnenhyApplyCodec(oEndContainer, fCodec, oSelection, null, iEndOffset, oEditor);
        }
      }

      // now process the in-between TEXT_NODEs
      oTreeWalker = document.createTreeWalker(oRange.commonAncestorContainer,
                                              NodeFilter.SHOW_TEXT,
                                              null,
                                              true);
      if (oTreeWalker)
      { // process all text nodes after the start node and before the final node
        for (oNextNode = oTreeWalker.nextNode(); oNextNode; )
        { // MnenhyApplyCodec may destroy oNextNode, so get the next node before changing
          oThisNode = oNextNode;
          oNextNode = oTreeWalker.nextNode();

          // select whole text to exclude boundaries
          var oCursorRange = document.createRange();
          oCursorRange.setStart(oThisNode, 0);
          oCursorRange.setEnd  (oThisNode, oThisNode.nodeValue.length);
          if (    (oCursorRange.compareBoundaryPoints(Range.START_TO_START, oRange) >= 0)
               && (oCursorRange.compareBoundaryPoints(Range.END_TO_END,     oRange) <= 0))
          {
            MnenhyApplyCodec(oThisNode, fCodec, oSelection, null, null, oEditor);
          }
          oCursorRange.detach();
        } // for oNextNode
      } // if oTreeWalker

      // recalculate iEndOffsetRev
      if (iEndOffsetRev < 0)
        iEndOffsetRev = oEndParent.childNodes[iEndIndex].nodeValue.length;
    } // different boundary containers or not text nodes

    // restore selection (need to recompute end offset in text nodes)
    oSelection.collapse(oStartParent.childNodes[iStartIndex], iStartOffset);
    oSelection.extend  (oEndParent  .childNodes[iEndIndex  ], iEndOffsetRev);
  }
  else
  { // no selection, so do it for all text children of the oRootElement
    oTreeWalker = document.createTreeWalker(oRootElement,
                                            NodeFilter.SHOW_TEXT,
                                            null,
                                            true);
    if (oTreeWalker)
    {
      for (oNextNode = oTreeWalker.nextNode(); oNextNode; )
      {
        oThisNode = oNextNode;
        oNextNode = oTreeWalker.nextNode();
        MnenhyApplyCodec(oThisNode, fCodec, oSelection, null, null, oEditor);
      }
    }
  }
}


//
// codecs
//
// codecs use arrays for performance reasons


// ROT-13
// unknown combinations are ignored
function MnenhyCodecROT13(sContent)
{
  var as = [];
  for (var i = 0; i < sContent.length; ++i)
  {
    var c = sContent[i];
    as[as.length] = (c in gasROT13) ? gasROT13[c] : c;
  }
  return as.join("");
}


// Base64
// if encoding/decoding fails, return old content
function MnenhyEncBase64(sContent)
{
  try
  {
    return btoa(sContent);
  }
  catch(e)
  {
    dump(e+"\n");
    return sContent;
  }
}
function MnenhyDecBase64(sContent)
{ // only A-Za-z0-9+/= are allowed
  const creAllowed    =  /[A-Za-z0-9\+\/=]/;
  const creNotAllowed = /[^A-Za-z0-9\+\/=]/;
  var as   = [];
  for (var i = 0; i < sContent.length;)
  { // search for the next not allowed char
    var p = sContent.substr(i).search(creNotAllowed);
    if (p < 0)
      p = sContent.length;
    // translate allowed part
    var s = sContent.substring(i, i + p);
    try
    {
      as[as.length] = atob(s);
    }
    catch(e)
    {
      dump(e+"\n");
      as[as.length] = s;
    }
    i += p;

    // process unallowed chars
    // search for the next allowed char
    p = sContent.substr(i).search(creAllowed);
    if (p < 0)
      p = sContent.length;
    // copy not allowed part
    as[as.length] = sContent.substring(i, i + p);
    i += p;
  } // for all sContent chars
  return as.join("");
}


// encodeURI / decodeURI
function MnenhyEncEscape(sContent)
{
  return encodeURI(sContent);
}
function MnenhyDecUnescape(sContent)
{
  return decodeURI(sContent);
}


// uuencode / uudecode
function MnenhyEncUUEncode(sContent)
{
  var as = [];
  for (var i = 0; i < sContent.length; i += 3)
  {
    var iA = sContent.charCodeAt(i);
    var iB = sContent.charCodeAt(i+1);
    if (isNaN(iB))
      iB = 0;
    var iC = sContent.charCodeAt(i+2);
    if (isNaN(iC))
      iC = 0;
    as[as.length] = String.fromCharCode(0x20 + (( iA >> 2                     ) & 0x3F),
                                        0x20 + (((iA << 4) | ((iB >> 4) & 0xF)) & 0x3F),
                                        0x20 + (((iB << 2) | ((iC >> 6) & 0x3)) & 0x3F),
                                        0x20 + (( iC                          ) & 0x3F));
  }
  return as.join("");
}
function MnenhyUUDecodeBuffer(aBuffer, aTarget)
{ // helper function for MnenhyDecUUDecode;
  // decoded 0x00-Bytes will be *ignored*,
  // so this implementation does *not* suffice for real binary data
  var iA = ((aBuffer[1] >> 4) | (aBuffer[0] << 2)) & 0xFF;
  var iB = ((aBuffer[2] >> 2) | (aBuffer[1] << 4)) & 0xFF;
  var iC = ((aBuffer[3]     ) | (aBuffer[2] << 6)) & 0xFF;
  if (iA)
    aTarget[aTarget.length] = String.fromCharCode(iA);
  if (iB)
    aTarget[aTarget.length] = String.fromCharCode(iB);
  if (iC)
    aTarget[aTarget.length] = String.fromCharCode(iC);
}
function MnenhyDecUUDecode(sContent)
{ // decoded 0x00-Bytes will be *ignored*,
  // so this implementation does *not* suffice for real binary data
  // other unknown bytes are tunneled through
  var as  = [];
  var asb = []; // decoding buffer
  var asp = 0;  // next valid element in buffer

  for (var i = 0; i < sContent.length; ++i)
  {
    var is  = sContent.charCodeAt(i) - 0x20;
    var bnv = ((is < 0x00) || (is > 0x3F));
    if (bnv)
    { // invalid char, enforce decoding now
      for (;asp < 4; ++asp)
        asb[asp] = 0;
    }
    else
    { // store valid char
      asb[asp++] = is;
    }
    if (asp == 4)
    { // enough data or invalid char
      MnenhyUUDecodeBuffer(asb, as);
      asp = 0;
      if (bnv)
        as[as.length] = sContent[i];
    }
  }
  if (asp)
  {
    for (;asp < 4; ++asp)
      asb[asp] = 0;
    MnenhyUUDecodeBuffer(asb, as);
  }
  return as.join("");
}


// simply reverse the string
function MnenhyCodecReverse(sContent)
{
  var as = [];
  for (var i = sContent.length - 1; i >= 0; --i)
    as[as.length] = sContent[i];
  return as.join("");
}


// Kenny-speak
// unknown combinations are tunneled through
function MnenhyEncKenny(sContent)
{
  var as = [];
  var s  = "";
  for (var j = 0; j < sContent.length; ++j)
  {
    s = sContent[j];
    as[as.length] = (s in gasKenny) ? gasKenny[s] : s;
  }
  return as.join("");
}
function MnenhyDecKenny(sContent)
{
  var as = [];
  var s  = "";
  for (var j = 0; j < sContent.length; j += s.length)
  {
    s = sContent.substr(j, 3);    // Kenny generated three chars
    as[as.length] = (s in gasKenny) ? gasKenny[s] : (s = s[0]);
  }
  return as.join("");
}


// Morse code
// unknown characters are dropped
// unknown combinations and whitespaces are spaced
// case is lost
function MnenhyEncMorse(sContent)
{
  var as = [];
  sContent = sContent.toLowerCase();
  for (var j = 0; j < sContent.length; ++j)
  {
    var s = sContent[j];
    switch (s)
    { // handle special chars
      case ".": s = "dot";   break;
      case "-": s = "minus"; break;
    }
    // unknown characters are dropped
    if (s in gasMorse)
      as[as.length] = gasMorse[s] + " ";
  }
  return as.join("");
}
function MnenhyDecMorse(sContent)
{ // only [.- ] are allowed
  const creAllowed    =  /[\.\-\ \n\r\f\t\v]/;
  const creNotAllowed = /[^\.\-\ \n\r\f\t\v]/;
  var as   = [];
  for (var i = 0; i < sContent.length;)
  { // search for the next not allowed char
    var p = sContent.substr(i).search(creNotAllowed);
    if (p < 0)
      p = sContent.length;
    // translate allowed part
    // if sContent's last char is a space, split generates a bogus element
    var sc = sContent.substring(i, i + p).replace(/ $/,"");
    var ac = sc.split(" ");
    for (var j = 0; j < ac.length; ++j)
    {
      var s = ac[j];
      // unknown combinations are spaces
      as[as.length] = (s in gasMorse) ? gasMorse[s] : " ";
    }
    i += p;

    // process unallowed chars
    // search for the next allowed char
    p = sContent.substr(i).search(creAllowed);
    if (p < 0)
      p = sContent.length;
    // copy not allowed part
    as[as.length] = sContent.substring(i, i + p);
    i += p;
  } // for all sContent chars
  return as.join("");
}


// Upper Case
// convert to upper case
function MnenhyEncUpperCase(sContent)
{
  return sContent.toUpperCase();
}


// Lower Case
// convert to lower case
function MnenhyEncLowerCase(sContent)
{
  return sContent.toLowerCase();
}
