/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Duesterloh.
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Duesterloh <mnenhy@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
// +-
// |    mnenhy-junk-stats.js
// |
// |    Written, forged, copied and conceived by Karsten Duesterloh <mnenhy@tprac.de>
// +-

// globals
var giMinRefCount = parseInt(gMnenhyRDF.GetItemValue("threshold","junk","general"), 10);
var grJunkTable   = [];
var goJunkTable   = null; // the DOM object

// functions

// get the training.dat
function GetJunkStatFile()
{
  var oDirServProp = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
  var sBaseDir     = oDirServProp.get("ProfD", Components.interfaces.nsIFile);
  var oFile = new CFile(sBaseDir.path); // dir
  oFile.append("training.dat");         // + name
  return oFile;
}

// read the content of the training.dat into local string
function GetStream(oFile)
{
  if (oFile && oFile.exists())
  {
    var IOS_CTR = "@mozilla.org/network/io-service;1";
    var nsIIOService = Components.interfaces.nsIIOService;

    var BUFIS_CTR = "@mozilla.org/network/buffered-input-stream;1";
    var nsIBufferedInputStream = Components.interfaces.nsIBufferedInputStream;

    var BINIS_CTR = "@mozilla.org/binaryinputstream;1";
    var nsIBinaryInputStream = Components.interfaces.nsIBinaryInputStream;

    var ios = Components.classes[IOS_CTR].getService(nsIIOService);
    var uri = ios.newFileURI(oFile);

    var inputStream = ios.newChannelFromURI(uri).open();
    var bufferedInputStream = Components.classes[BUFIS_CTR].createInstance(nsIBufferedInputStream);
    bufferedInputStream.init(inputStream, oFile.fileSize);

    var binaryInputStream = Components.classes[BINIS_CTR].createInstance(nsIBinaryInputStream);
    binaryInputStream.setInputStream(bufferedInputStream);

    return binaryInputStream;
  }
  return null;
}


// compute the bayesian junk probality of a word
// (see  nsBayesianFilter::classifyMessage in
//  /seamonkey/source/mailnews/extensions/bayesian-spam-filter/src/nsBayesianFilter.cpp)
// return a percent value
function GetJunkProb(iGood, iGoodMsg, iEvil, iEvilMsg)
{
  var p  = 0.4;       // default probality is 40%
  var ig = 2 * iGood; // weigh good tokens count twice

  if (ig + iEvil > 5)
  {
    var db = Math.min(1.0, iEvil / iEvilMsg);
    var dg = Math.min(1.0, ig    / iGoodMsg);
    p = db / (db + dg);
    p = Math.max(0.01, Math.min(0.99, p));
  }
  return 100*p;
}


// read token data into array, but ignore tokens with both refcounts below threshold
// the associative array aHash is used to map tokens to indices
function ProcessTokens(aTable, aHash, is, iTokens, bGood, iGoodMsg, iEvilMsg)
{ 
  dump("Mnenhy::ProcessTokens")

  // conversion to Unicode
  // http://lxr.mozilla.org/mozilla/source/intl/uconv/idl/nsIScriptableUConv.idl
  var oUTFConv     = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
  oUTFConv.charset = "UTF-8";

  var iRefCount, iTokenLen;
  
  for (var i = 0; i < iTokens; ++i)
  {
    if(i%1000==0)
      dump(".")

    // read token data (add space to avoid problems with keywords)
    iRefCount = is.read32();
    iTokenLen = is.read32();
    var oToken = {sToken: is.readBytes(iTokenLen)+" ",
                  iGood:  0, 
                  iEvil:  0, 
                  iProb:  0};
    // most tokens are in UTF-8 (but not all, so ConvertToUnicode may vomit :( )
    try{    
      oToken.sToken = oUTFConv.ConvertToUnicode(oToken.sToken);
    }catch(ignore){}

    if (bGood)
    { // good tokens are new or ignored
      if (iRefCount >= giMinRefCount)
      { // new good token
        oToken.iGood = iRefCount;
        oToken.iProb = GetJunkProb(iRefCount, iGoodMsg, 0, iEvilMsg);
        aHash[oToken.sToken] = aTable.length;
        aTable.push(oToken);
      }
    }
    else
    { // evil tokens are known, unimportant or new
      if (oToken.sToken in aHash)
      { // add evil refcount to known token and recompute ratio
        oToken = aTable[aHash[oToken.sToken]];
        oToken.iEvil = iRefCount;
        oToken.iProb = GetJunkProb(oToken.iGood, iGoodMsg, iRefCount, iEvilMsg);
      }
      else if (iRefCount >= giMinRefCount)
      { // new evil token
        oToken.iEvil = iRefCount;
        oToken.iProb = GetJunkProb(0, iGoodMsg, iRefCount, iEvilMsg);
        aHash[oToken.sToken] = aTable.length;
        aTable.push(oToken);
      }
    }

  } // for
  dump("=" + iTokens + "\n")
}


// some sort functions
function SortByToken(o1,o2)
{
  if (o1.sToken < o2.sToken)
    return -1;
  if (o1.sToken > o2.sToken)
    return  1;
  return 0;
}
function SortByGood(o1,o2)
{
  return o1.iGood - o2.iGood;
}
function SortByEvil(o1,o2)
{
  return o1.iEvil - o2.iEvil;
}
function SortByRatio(o1,o2)
{
  return o1.iProb - o2.iProb;
}


// create html junk table
function SetJunkTable(fSortFunction, bDescending)
{ // sort the table
  try
  {
    if (fSortFunction)
    {
      grJunkTable.sort(fSortFunction);
      if (bDescending)
        grJunkTable.reverse();
    }
    // build it
    var sTable = "";

    for (var i = 0; i < grJunkTable.length; ++i)
      sTable += "<tr><td class='JunkCol0'>" + String(i+1) + "</td><td class='JunkCol1'>"
                        + grJunkTable[i].sToken           + "</td><td>"
                        + grJunkTable[i].iGood            + "</td><td>"
                        + grJunkTable[i].iEvil            + "</td><td>"
                        + grJunkTable[i].iProb.toFixed(2) + " %</td></tr>\n";
    // show it
    SetInnerXML(goJunkTable, sTable);
  }
  catch(e)
  {
    alert("SetJunkTable("+fSortFunction+","+bDescending+")\n"+e);
  }
}


// create XHTML-DOM nodes from string and attach them to oParent
function AddInnerXML(oParent, sXML)
{ // set new
  var oDOMParser = new DOMParser();
  var oNodes     = oDOMParser.parseFromString("<dummy xmlns='http://www.w3.org/1999/xhtml'>"+sXML+"</dummy>", "text/xml").firstChild.childNodes;
  for (var i = 0; i < oNodes.length; ++i)
    oParent.appendChild(oNodes[i]);
}


// create XHTML-DOM nodes from string and attach only them to oNode
function SetInnerXML(oNode, sXML)
{ // delete child nodes first
  while (oNode.lastChild)
    oNode.removeChild(oNode.lastChild);
  // set new
  AddInnerXML(oNode, sXML);
}


// read training.dat
//
//    [0xFEEDFACE]
//    [number good messages][number bad messages]
//    [number good tokens]
//    [refcount][length of token]token
//    ....
//    [number bad tokens]
//    [refcount][length of token]token
//
// return prolog html text
function ReadJunkStats()
{
  // start time counter
  var iStartTime = new Date().getTime()/1000;

  // open training.dat
  var oTrainingDat = GetJunkStatFile();
  var is = GetStream(oTrainingDat);
  if (!is)
    return;

  // check magic number
  var iMagicNumber = is.read32();
  if (iMagicNumber != 0xFEEDFACE)
    return;

  // set ham'n'spam numbers
  var iGood  = is.read32();
  var iEvil  = is.read32();
  var iMsgs  = iGood + iEvil;
  var iGoodP = Math.round(iGood/iMsgs*100);
  var iEvilP = 100-iGoodP;

  var sProlog = GetMnenhyStr("MnDialog_Junk_Prolog");
  sProlog = sProlog.replace("%m",  String(iMsgs ));
  sProlog = sProlog.replace("%h",  String(iGood ));
  sProlog = sProlog.replace("%hp", String(iGoodP));
  sProlog = sProlog.replace("%s",  String(iEvil ));
  sProlog = sProlog.replace("%sp", String(iEvilP));
  sProlog = sProlog.replace("%b",  String(oTrainingDat.fileSize));

  // read good and evil tokens
  var aHash = []; // pseudo hash table
  var iGoodTokens = is.read32();
  ProcessTokens(grJunkTable, aHash, is, iGoodTokens, true,  iGood, iEvil);
  var iEvilTokens = is.read32();
  ProcessTokens(grJunkTable, aHash, is, iEvilTokens, false, iGood, iEvil);
  aHash = [];     // thou art not needed no more ;-)

  // stop time counter
  var iEndTime  = new Date().getTime()/1000;
  var iDuration = parseInt(100*(iEndTime - iStartTime),10)/100;

  // set good'n'evil numbers
  var iTokens  = iGoodTokens + iEvilTokens;
  var iGoodTP  = parseInt(iGoodTokens/iTokens*100,10);
  var iEvilTP  = 100-iGoodTP;
  var iIgnored = iTokens - grJunkTable.length;

  sProlog = sProlog.replace("%d",  String(iDuration  ));
  sProlog = sProlog.replace("%j",  String(iTokens    ));
  sProlog = sProlog.replace("%g",  String(iGoodTokens));
  sProlog = sProlog.replace("%gp", String(iGoodTP    ));
  sProlog = sProlog.replace("%e",  String(iEvilTokens));
  sProlog = sProlog.replace("%ep", String(iEvilTP    ));
  sProlog = sProlog.replace("%i",  String(iIgnored   ));
  sProlog = sProlog.replace("%t",  String(giMinRefCount));

  return sProlog;
}


// start the reading of training.dat in a separate thread
// otherwise Mozilla will freeze until computation is finished
function InitJunkStats()
{
  // check junk threshold
  if (!giMinRefCount)
  {
    var oTrainingDat = GetJunkStatFile();
    giMinRefCount = parseInt(oTrainingDat.fileSize/1000,10);
  }

  // prepare the iframe
  goJunkTable = document.getElementById('mnenhyJunkTable');
  
  SetInnerXML(document.getElementById("mnenhyJunkProlog"), GetMnenhyStr("MnDialog_Junk_Loading"));

  // freezes Mozilla during the calculation
  var sProlog = ReadJunkStats();

  SetJunkTable();
  SetInnerXML(document.getElementById("mnenhyJunkProlog"), sProlog);
}
