/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Duesterloh.
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Duesterloh <mnenhy@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
// +-
// |    mnenhy-pref-header.js
// |
// |    Written, forged, copied and conceived by Karsten Duesterloh <mnenhy@tprac.de>
// +-

// |
// | constants
// |

// container names
const csBaseContainer    = "header"
const csHeaderContPrefix = csBaseContainer + ":";
const csKnown            = "known";
const csKnownHeaderCont  = csHeaderContPrefix + csKnown;
const csFilter           = "filter";
const csFilterPref       = "mailnews.customHeaders";
const csCompose          = "compose";
const csComposePref      = "mail.compose.other.header";
const csCustom           = "custom";
const csFormat           = "format";
const csUseToggle        = "useToggle";

const casHeaderVars = ["gkiOutputText","gkiOutputMail","gkiOutputNews","gkiOutputMID"];
const casHeader     = [GetMnenhyStr("MnJSXUL_header_text"),
                       GetMnenhyStr("MnJSXUL_header_mail"),
                       GetMnenhyStr("MnJSXUL_header_news"),
                       GetMnenhyStr("MnJSXUL_header_mid")];

// idefixes (wuff!)
const csListPrefix    = "mnenhy-lh-";
const csDropPrefix    = "mnenhy-dd-";
const csCheckPostfix  = "-chk";
const csFormatPostfix = "-fmt";


// |
// | global objects (will be set in InitMnenhyPrefHeader)
// |
var goListbox      = null;
var goDropdown     = null;
var goDropdownList = null;


// |
// | vars
// |

var gsCurrentListId = null;


// |
// | functions
// |
function InitMnenhyPrefHeader(iDelay)
{ // mnenhy's rdf datasource needs to be ready
  if (!gMnenhyRDF.mDataSourceLoaded || !goListbox)
  {
    dump("InitMnenhyPrefHeader("+iDelay+");\n");
    SetWaitCursor();
    if (iDelay>0)
      setTimeout("InitMnenhyPrefHeader("+(iDelay - 1)+");", 50);
    else
      alert(GetMnenhyStr("MnErr_Slow"));
    goListbox = document.getElementById("mnenhyHeaderListbox");
  }
  else
  { // init global objects
    dump("InitMnenhyPrefHeader() ok\n");

    goListbox      = document.getElementById("mnenhyHeaderListbox");
    goDropdown     = document.getElementById("mnenhyHeaderDropdown");
    goDropdownList = document.getElementById("mnenhyHeaderDropdownList");

    // get the known headers and prepare check array
    var asKnownHeaders = gMnenhyRDF.GetNodeList(csKnownHeaderCont, true);
    var abCheck = [];
    for (var sHeader in asKnownHeaders)
    {
      abCheck[sHeader]=true;
    }

    // get the preference headers
    // -- get the filter headers
    var sPrefHeaderList = MnenhyGetCleanStringPref(csFilterPref).replace(/:/g,",");
    // -- get the compose headers
    sPrefHeaderList += "," + MnenhyGetCleanStringPref(csComposePref);
    var asPrefHeaderList = sPrefHeaderList.match(/[^,]+/g);

    // now check if they're already known
    for (var iIndex in asPrefHeaderList)
    {
      var sPrefHeader = asPrefHeaderList[iIndex];
      var bValue = abCheck[sPrefHeader];
      if (!IsTrue(bValue))
      { // not known, so add to abCheck...
        abCheck[sPrefHeader]=true;
        // ...and to rdf
        var asItems = {format:"gkiOutputText"};                     // add as text item
        gMnenhyRDF.AddNode(sPrefHeader,csKnownHeaderCont,asItems);  // add to known
      }
    }

    // show all known headers in the listbox
    FillListbox();

    // fill dropdownbox with header list names
    // - standard items
    AddMenuItem(goDropdown, csDropPrefix+csFilter,  'SelectMnenhyHeaderList("'+csFilter+'")',
                GetMnenhyStr("MnJSXUL_header_list_filter"));
    AddMenuItem(goDropdown, csDropPrefix+csCompose, 'SelectMnenhyHeaderList("'+csCompose+'")',
                GetMnenhyStr("MnJSXUL_header_list_compose"));
    // - custom items
    var iLen = CustomHeaderViewCount()
    if (iLen)
    {
      goDropdown.appendChild(document.createElement("menuseparator"));

      var asHeaderLists = gMnenhyRDF.GetNodeList(csBaseContainer);
      iLen = asHeaderLists.length;
      // get all custom lists, but ignore the known headers
      for (var iList = 0; iList < iLen; ++iList)
      {
        var node = asHeaderLists[iList];
        if (node != csKnown)  // ignore some header lists; currently only one
        {
          var prettyname = gMnenhyRDF.GetItemValue("name",node,csBaseContainer);
          AddMenuItem(goDropdown, csDropPrefix+node, "SelectMnenhyHeaderList('" + node + "')",
                      prettyname, null);
        }
      }
    }
    UnsetWaitCursor();
  }
}


// show all known headers in the listbox
function FillListbox()
{ // empty the listbox
  EmptyListbox(goListbox);
  // refill it
  var asKnownHeaders = gMnenhyRDF.GetNodeList(csKnownHeaderCont);
  asKnownHeaders.sort();
  var iLen = asKnownHeaders.length;
  for (var iHeader = 0; iHeader < iLen; ++iHeader)
  {
    var sh = asKnownHeaders[iHeader];
    // get the style classes for this header:
    var sClass = "";
    var sPrettyHeader = Capitalize(sh);
    // -- normal = bold
    if (IsTrue(gMnenhyRDF.GetItemValue("hl_normal",sh,csKnownHeaderCont)))
      sClass += " mnenhy-lh-bold";
    // -- filter = italics
    if (IsTrue(gMnenhyRDF.GetItemValue("hl_filter",sh,csKnownHeaderCont)))
      sClass += " mnenhy-lh-italic";
    // -- compose = small caps
    if (IsTrue(gMnenhyRDF.GetItemValue("hl_compose",sh,csKnownHeaderCont)))
    { // Mozilla won't do small-caps in listboxes, so we've got to fake it
      // we can't underline, since this underlines white space also
      sPrettyHeader = sPrettyHeader.toUpperCase();
    }

    AddListItem(goListbox, csListPrefix+sh,
                sPrettyHeader, sClass,
                casHeader[eval(gMnenhyRDF.GetItemValue(csFormat,sh,csKnownHeaderCont))], null,
                true, false);
  }
}



// create a listitem in goListbox with id=sId and two cells sLabel and sType
// bDisabled can be used to disable the listitem, bChecked governs the checkbox default state
function AddListItem(oListbox, sId, sLabel, sLabelStyleClass, sType, sTypeStyleClass, bDisabled, bChecked)
{
  var oListitem = document.createElement("listitem");
  oListitem.setAttribute("id",       sId);

  var sCheckId  = sId + csCheckPostfix;
  var sFormatId = sId + csFormatPostfix;

  var oListcell = document.createElement("listcell");
  oListcell.setAttribute("id",    sCheckId);
  oListcell.setAttribute("type",  "checkbox");
  oListcell.setAttribute("label", sLabel);
  oListcell.setAttribute("class", sLabelStyleClass);
  oListitem.appendChild(oListcell);

  oListcell = document.createElement("listcell");
  oListcell.setAttribute("id",    sFormatId);
  oListcell.setAttribute("label", sType);
  oListcell.setAttribute("class", sTypeStyleClass);
  oListitem.appendChild(oListcell);

  oListbox.appendChild(oListitem);

  SetListcell(sCheckId,  bChecked, bDisabled);
  SetListcell(sFormatId, bChecked, bDisabled);

  return oListitem;
}


// create a menuitem in goDropdown with id=sId and label sLabel
function AddMenuItem(oDropdown, sId, sCommand, sLabel, sLabelStyle)
{
  var oMenuitem = document.createElement("menuitem");
  oMenuitem.setAttribute("id",       sId);
  oMenuitem.setAttribute("label",    sLabel);
  oMenuitem.setAttribute("oncommand",sCommand);
  oDropdown.appendChild(oMenuitem);
  return oMenuitem;
}


// set 'checked' and 'disabled' of a listcell
function SetListcell(sListcell, bChecked, bDisabled)
{
  try
  {
    var oCell = document.getElementById(sListcell);
    oCell.setAttribute("checked", String(IsTrue(bChecked)));
    oCell.setAttribute("disabled",String(IsTrue(bDisabled)));
  }
  catch(ignored){}
}


// clear all the checkboxes of the header listbox
function ClearListCheckboxes(oListbox)
{
  var iLen = oListbox.getRowCount();
  for (var iRow = 0; iRow < iLen; ++iRow)
  {
    var oItem = oListbox.getItemAtIndex(iRow);
    SetListcell(oItem.id + csCheckPostfix);
    SetListcell(oItem.id + csFormatPostfix);
  }
}


// set those checkboxes of the header listbox that are specified in the array
// if bDisable is specified, the listitem attribute will be set accordingly
function SetListCheckboxes(sListitemPrefix, asHeaderList, bDisable)
{
  var iLen = asHeaderList.length;
  for (var iHeader = 0; iHeader < iLen; ++iHeader)
  {
    var sListitem = sListitemPrefix + asHeaderList[iHeader];
    SetListcell(sListitem + csCheckPostfix,  true, bDisable);
    SetListcell(sListitem + csFormatPostfix, true, bDisable);
  }
}


// mark headers used by current list
function SelectMnenhyHeaderList(sNode)
{ // first clear the listbox checkboxes
  ClearListCheckboxes(goListbox);
  // then set new status
  gsCurrentListId = sNode;
  switch(sNode)
  {
    case csFilter:
      { // set headers from pref
        var sPrefHeaderList = MnenhyGetCleanStringPref(csFilterPref);
        var asPrefHeaderList = sPrefHeaderList.match(/[^:]+/g);
        SetListCheckboxes(csListPrefix, asPrefHeaderList, false);
        // set default headers
        // must be set after pref to overrule double entries
        var asKnownHeaderList = gMnenhyRDF.GetNodeList(csKnownHeaderCont, false, "hl_filter", true);
        SetListCheckboxes(csListPrefix, asKnownHeaderList, true);
      }
      break;

    case csCompose:
      { // set headers from pref
        var sPrefHeaderList2 = MnenhyGetCleanStringPref(csComposePref);
        var asPrefHeaderList2 = sPrefHeaderList2.match(/[^,]+/g);
        SetListCheckboxes(csListPrefix, asPrefHeaderList2, false);
        // set default headers
        // must be set after pref to overrule double entries
        var asKnownHeaderList2 = gMnenhyRDF.GetNodeList(csKnownHeaderCont, false, "hl_compose", true);
        SetListCheckboxes(csListPrefix, asKnownHeaderList2, true);
      }
      break;

    default:
      { // set custom headers
        var asHeaderList = gMnenhyRDF.GetNodeList(csHeaderContPrefix + sNode);
        SetListCheckboxes(csListPrefix, asHeaderList);
      }
  }
}


// listitem click event: check for checkbox
function CheckListitemClick(event)
{ // left click in checkbox?
  // +- listitem   <-- click event happens here
  // |  +- listcell
  // |  |  +- xul:image
  // |  |  +- xul:label
  // |  +- listcell
  // |     +- xul:label
  // +- listitem
  // ...
  try
  {
    if (    (!event.button)
         && (event.clientX < event.originalTarget.boxObject.firstChild.boxObject.firstChild.boxObject.nextSibling.boxObject.x)
       )
    {
      ToggleHeaderItem(event.originalTarget);
    }
  }
  catch(ignore)
  {}
}


// listitem keydown event: check for VK_SPACE
function CheckListitemKeyDown(event)
{
  if (event.keyCode == 32)  // Space in listbox!
  {
    ToggleHeaderItem(goListbox.selectedItem);
  }
}


// toggle the header checkbox if not disabled
function ToggleHeaderItem(target)
{
  var oCellNode = target.childNodes[0];
  var sDisabled = oCellNode.getAttribute("disabled");
  if (!IsTrue(sDisabled))
  {
    var sHeader = target.id.substr(csListPrefix.length);

    var sChecked  = oCellNode.getAttribute("checked");
    if( sChecked != "true" )
    { // add this header to current listbox
      oCellNode.setAttribute("checked","true");
      // add to list
      SetHeaderState(gsCurrentListId,sHeader,true);
    }
    else
    { // remove this header from current list
      oCellNode.setAttribute("checked","false");
      // remove from list
      SetHeaderState(gsCurrentListId,sHeader,false);
    }
  }
}


// (dis)activate a header in rdf or pref
function SetHeaderState(sCurrentListId, sHeader, bState)
{
  switch (sCurrentListId)
  {
    case csFilter:
      { // read pref
        var sPref = MnenhyGetCleanStringPref(csFilterPref).replace(/:/g,",");
        // change it
        sPref = SetPrefHeaderState(sPref, bState, sHeader);
        // write pref
        sPref = sPref.replace(/,/g,": ");
        MnenhySetPref(csFilterPref,"char",Capitalize(sPref));
      }
      break;

    case csCompose:
      { // read pref
        var sPref2 = MnenhyGetCleanStringPref(csComposePref);
        // change it
        sPref2 = SetPrefHeaderState(sPref2, bState, sHeader);
        // write pref
        MnenhySetPref(csComposePref,"char",Capitalize(sPref2));
      }
      break;

    default:
      {
        var sCont = csHeaderContPrefix + sCurrentListId;
        if (bState)
          gMnenhyRDF.AddNode(sHeader,sCont);
        else
          gMnenhyRDF.RemoveNode(sHeader,sCont);
      }
  }
  ForceHeaderRebuild();
}


// the pref string consists of comma-delimited values
// the changed string will be returned
// bActive controls setting (true) or deleting (false) the sValue
function SetPrefHeaderState(sPref, bActive, sValue)
{
  sPref  = "," + sPref  + ",";
  sValue = "," + sValue + ",";
  if (sPref.indexOf(sValue) != -1)
  { // value found
    if (!bActive)
    { // remove
      sPref = sPref.replace(sValue,",");  // remove item
    }
  }
  else
  { // value not found
    if (bActive)
    { // set
      sPref = sPref.replace(/,/,sValue);  // set as new first item
    }
  }
  return sPref.replace(/^,+|,+$/g, "");   // trim commas
}


// empty the listbox
function EmptyListbox(oListbox)
{
  if (oListbox.getRowCount())
  {
    var oThis = oListbox.getItemAtIndex(0);
    while(oThis)
    {
      if (oThis.nodeName=="listitem")
      {
        var oKill = oThis;
        oThis = oListbox.getNextItem(oThis,1);
        oListbox.removeChild(oKill);
      }
      else
      {
        oThis = oListbox.getNextItem(oThis,1);
      }
    }
  }
}


function AddMnenhyList()
{ // get new list name
  var oName = {value:"",
               sTitle:GetMnenhyStr("MnDialog_header_addlist_title"),
               sLabelValue:GetMnenhyStr("MnDialog_header_addlist_label"),
               sTextboxValue:""};
  window.openDialog("chrome://mnenhy/content/mnenhy-pref-header-input.xul",
                    "mnenhyPrefHeaderInput",
                    "chrome,modal,centerscreen",
                    oName);
  if (oName.value && oName.value.length)
  { // find first empty custom list
    var iList = 0;
    while (gMnenhyRDF.ExistNode(csCustom+iList,csBaseContainer))
      ++iList;
    var sInternalName = csCustom+iList;
    // add list name to listbox
    if (!CustomHeaderViewCount())
      goDropdown.appendChild(document.createElement("menuseparator"));
    AddMenuItem(goDropdown, csDropPrefix+sInternalName, "SelectMnenhyHeaderList('" + sInternalName + "')",
                oName.value, null);
    // add list to rdf
    gMnenhyRDF.AddContainer(sInternalName, "Seq", csBaseContainer);
    gMnenhyRDF.ChangeItemValue(oName.value, "name", sInternalName, csBaseContainer);
    // activate the new list
    goDropdownList.selectedIndex = goDropdown.childNodes.length - 1;
    SelectMnenhyHeaderList(sInternalName);
  }
}


function RenameMnenhyList()
{ // get old list name
  var oSelItem = goDropdownList.selectedItem;
  if (oSelItem)
  { // we need an item to rename...
    var sInternalName = oSelItem.id.replace(csDropPrefix,"");
    // internal name must be customxxx
    if (!sInternalName.indexOf(csCustom))
    {
      var oName = {value:"",
                   sTitle:GetMnenhyStr("MnDialog_header_renamelist_title"),
                   sLabelValue:GetMnenhyStr("MnDialog_header_addlist_label"),
                   sTextboxValue:oSelItem.label};
      window.openDialog("chrome://mnenhy/content/mnenhy-pref-header-input.xul",
                        "mnenhyPrefHeaderInput",
                        "chrome,modal,centerscreen",
                        oName);
      if (oName.value && oName.value.length)
      { // rename!
        oSelItem.label = oName.value;
        gMnenhyRDF.ChangeItemValue(oName.value, "name", sInternalName, csBaseContainer);
        var i = goDropdownList.selectedIndex;
        goDropdownList.selectedIndex = -1;
        goDropdownList.selectedIndex = i;  // refresh dropdownbox
      }
    }
  }
}


function RemoveMnenhyList()
{
  var oSelItem = goDropdownList.selectedItem;
  if (oSelItem)
  { // we need an item to kill...
    var sInternalName = oSelItem.id.replace(csDropPrefix,"");
    // internal name must be customxxx
    if (!sInternalName.indexOf(csCustom))
    {
      var oName = {value:"",
                   sTitle:GetMnenhyStr("MnDialog_header_removelist_title"),
                   sLabelValue:GetMnenhyStr("MnDialog_header_removelist_label").replace("%s",oSelItem.label),
                   bHideTextbox:true};
      window.openDialog("chrome://mnenhy/content/mnenhy-pref-header-input.xul",
                        "mnenhyPrefHeaderInput",
                        "chrome,modal,centerscreen",
                        oName);
      if (IsTrue(oName.value))
      { // remove from rdf
        gMnenhyRDF.RemoveContainer(csHeaderContPrefix + sInternalName);
        gMnenhyRDF.RemoveNode(sInternalName, csBaseContainer);
        // remove from dropdownbox
        var i = goDropdownList.selectedIndex;
        goDropdownList.removeItemAt(i);
        if (goDropdown.childNodes.length == 3)
        { // no custom headers left
          goDropdownList.removeItemAt(2);  // separator
          i=1;
        }
        // refresh dropdownbox
        if (i < goDropdown.childNodes.length)
          goDropdownList.selectedIndex = i;
        else
          goDropdownList.selectedIndex = i - 1;
        SelectMnenhyHeaderList(goDropdownList.selectedItem.id.replace(csDropPrefix,""));
      }
    }
  }
}


function ForceHeaderRebuild(sHeaderName)
{ // force rebuild of all header lists in all mail windows
  var oWM = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                      .getService(Components.interfaces.nsIWindowMediator);
  var oWinList = oWM.getEnumerator(null);
  while (oWinList.hasMoreElements())
  {
    var oWin = oWinList.getNext();
    if ("giHeaderView" in oWin)
    {
      if (oWin.gExpandedHeaderView && (sHeaderName in oWin.gExpandedHeaderView))
      {
        var oBox = oWin.gExpandedHeaderView[sHeaderName].enclosingBox;
        if (oBox)
          oBox.parentNode.removeChild(oBox);
        delete oWin.gExpandedHeaderView[sHeaderName];
      }
      oWin.giHeaderView = oWin.cViewBriefDirty; // request header list rebuild
      oWin.MsgReload();                         // reload current message
    }
  }
}


function AddMnenhyHeader()
{ // get new header
  var oName = {value:           "",
               sTitle:          GetMnenhyStr("MnDialog_header_add_title"),
               sLabelValue:     GetMnenhyStr("MnDialog_header_add_label"),
               sTextboxValue:   "",
               bShowListbox:    true,
               sListLabelValue: GetMnenhyStr("MnDialog_header_add_listlabel"),
               saList:          casHeader,
               iListSelected:   0};
  window.openDialog("chrome://mnenhy/content/mnenhy-pref-header-input.xul",
                    "mnenhyPrefHeaderInput",
                    "chrome,modal,centerscreen",
                    oName);
  if (oName.value && oName.value.name && oName.value.name.length)
  {
    SetWaitCursor();
    var sFormat       = casHeaderVars[oName.value.format];
    var sInternalName = oName.value.name.toLowerCase();
    // add only if not already there
    if (!gMnenhyRDF.ExistNode(sInternalName, csHeaderContPrefix + csKnown))
    { // create new header in rdf
      var asItems = {format:sFormat};
      gMnenhyRDF.AddNode(sInternalName, csKnownHeaderCont, asItems);  // add to known
    }  
    // set useToggle mode
    var bUseToggle = (oName.value.format == gkiOutputMail);
    gMnenhyRDF.ChangeItemValue(bUseToggle, csUseToggle, sInternalName, csKnownHeaderCont);

    // refill listbox
    FillListbox(goListbox);
    if (goDropdownList.selectedItem)
      SelectMnenhyHeaderList(goDropdownList.selectedItem.id.replace(csDropPrefix, ""));
    // select new header
    var oItem = document.getElementById(csListPrefix + sInternalName);
    if (oItem)
    {
      goListbox.selectedItem = oItem;
      goListbox.focus();
    }
    ForceHeaderRebuild(sInternalName);
    UnsetWaitCursor();
  }
}


function RemoveMnenhyHeader()
{
  var oSelItem = goListbox.selectedItem;
  if (oSelItem)
  { // remove only non-disabled items
    var oChkCell = document.getElementById(oSelItem.id + csCheckPostfix);
    if (oChkCell && !IsTrue(oChkCell.getAttribute("disabled")))
    {
      var sInternalName = oSelItem.id.replace(csListPrefix,"");
      var oName = {value:       "",
                   sTitle:      GetMnenhyStr("MnDialog_header_remove_title"),
                   sLabelValue: GetMnenhyStr("MnDialog_header_remove_label").replace("%s",oChkCell.getAttribute("label")),
                   bHideTextbox:true};
      window.openDialog("chrome://mnenhy/content/mnenhy-pref-header-input.xul",
                        "mnenhyPrefHeaderInput",
                        "chrome,modal,centerscreen",
                        oName);
      if (IsTrue(oName.value))
      { // remove all references to this header
        SetWaitCursor();
        SetHeaderState(csFilter,  sInternalName, false)
        SetHeaderState(csCompose, sInternalName, false)
        var iMaxCustom = CustomHeaderViewCount();
        for (var i = 0; i < iMaxCustom; ++i)
          SetHeaderState(csCustom+i, sInternalName, false);
        // remove from known
        gMnenhyRDF.RemoveNode(sInternalName, csKnownHeaderCont);
        // refill listbox
        FillListbox(goListbox);
        if (goDropdownList.selectedItem)
          SelectMnenhyHeaderList(goDropdownList.selectedItem.id.replace(csDropPrefix, ""));
        ForceHeaderRebuild(sInternalName);
        UnsetWaitCursor();
      }
    }
  }
}


function EditMnenhyHeader()
{ // change the view format of the header
  var oSelItem = goListbox.selectedItem;
  if (oSelItem)
  {
    var sInternalName = oSelItem.id.replace(csListPrefix,"");
    var iFormat = eval(gMnenhyRDF.GetItemValue(csFormat, sInternalName, csKnownHeaderCont));
    var oName = {value:           "",
                 sTitle:          GetMnenhyStr("MnDialog_header_edit_title"),
                 sLabelValue:     GetMnenhyStr("MnDialog_header_edit_label"),
                 sTextboxValue:   sInternalName,
                 bShowListbox:    true,
                 bDisableTextbox: true,
                 sListLabelValue: GetMnenhyStr("MnDialog_header_add_listlabel"),
                 saList:          casHeader,
                 iListSelected:   iFormat};
    window.openDialog("chrome://mnenhy/content/mnenhy-pref-header-input.xul",
                      "mnenhyPrefHeaderInput",
                      "chrome,modal,centerscreen",
                      oName);
    if (oName.value && (oName.value.format != iFormat))
    {
      SetWaitCursor();
      gMnenhyRDF.ChangeItemValue(casHeaderVars[oName.value.format], csFormat, sInternalName, csKnownHeaderCont);
      // set useToggle mode
      var bUseToggle = (oName.value.format == gkiOutputMail);
      gMnenhyRDF.ChangeItemValue(bUseToggle, csUseToggle, sInternalName, csKnownHeaderCont);

      // refill listbox
      FillListbox(goListbox);
      if (goDropdownList.selectedItem)
        SelectMnenhyHeaderList(goDropdownList.selectedItem.id.replace(csDropPrefix, ""));
      // select new header
      var oItem = document.getElementById(csListPrefix + sInternalName);
      if (oItem)
      {
        goListbox.selectedItem = oItem;
        goListbox.focus();
      }
      ForceHeaderRebuild(sInternalName);
      UnsetWaitCursor();
    }
  }
}
